/**
 * 
  ASkeyboard.h	Definitions of "Hobo-nicola ASKeyboard" using Hobo-nicola keyboard and adapter library.
  Copyright (c) 2019 Takeshi Higasa
  
  This file is part of "Hobo-nicola keyboard and adapter library".

  "Hobo-nicola keyboard and adapter library" is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  "Hobo-nicola keyboard and adapter library" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Hobo-nicola keyboard and adapter library".  If not, see <http://www.gnu.org/licenses/>.
  
	Icluded in hoboNicola150.zip.  Aug. 10. 2021.
  

*/
#include <Arduino.h>
//#include "hobo_nicola.h"
// scan-code
#define ASKB_CTRL_L		0x40
#define ASKB_CTRL_R		0x4c
#define ASKB_SHIFT_L	0x43
#define ASKB_SHIFT_R	0x44
#define ASKB_GRPH		0x46	// GPRH
#define ASKB_ALT_R
#define ASKB_CAPS		0x45	// CAPS
#define ASKB_GUI_R

#define ASKB_KANA		0x47	// Ji
#define ASKB_EISU		0x48	// TAB(NFER)

#define ASKB_FN			0x4d	// red ALT.

// ASkeyboard OYAYUBI-keys
#define ASKB_LEFT_OYAYUBI	0x72		// HID Keyboard F23
#define ASKB_RIGHT_OYAYUBI	0x73		// HID Keyboard F24


#define SET_KANA_LED(a)  PORTB = a ? PORTB & ~0x40 : PORTB | 0x40
#define SET_CAPS_LED(a)  PORTE = a ? PORTE & ~0x40 : PORTE | 0x40
#define SET_NUML_LED(a)  PORTC = a ? PORTC & ~0x40 : PORTC | 0x40
#define SET_CXFR_LED(a)  PORTD = a ? PORTD & ~0x80 : PORTD | 0x80
#define SET_KBDL_LED(a)  PORTD = a ? PORTD & ~0x10 : PORTD | 0x10


#define TOGGLE_KANA_LED() (PINB |= 0x40)
#define TOGGLE_CAPS_LED() (PINE |= 0x40)
#define TOGGLE_NUML_LED() (PINC |= 0x40)
#define TOGGLE_CXFR_LED() (PIND |= 0x80)
#define TOGGLE_KBDL_LED() (PIND |= 0x10)


class keyboard_notify {
public:
	virtual void key_pressed(uint8_t k, uint8_t mod) {}; // notify key pressed with hid usage id.
	virtual void key_released(uint8_t k, uint8_t mod) {}; // notify key released with hid usage id.
	virtual void modifier_changed(uint8_t before, uint8_t after) {}; // notify any modifier-key is changed.
	virtual void raw_key_event(uint8_t scan, bool released = false) {};  // notify all key event with raw scancode.
	virtual void key_released_all() {}; // all keys are released.
	virtual void enter_setup() {};
};

class ASkeyboard {
public:
	static ASkeyboard* getInstance() {
		static ASkeyboard ask;
		return &ask;
	};
	bool begin(keyboard_notify* p = 0);
	void task(uint8_t hid_led_state = 0);

	bool is_nicola_disabled() const;	// dip-sw pos == JIS.
	void setup_led(bool on);
	bool dedicated_oyayubi() const;

private:
	uint8_t kbd_led_state;	// hid keyboard led.
	uint8_t modifier_state;
	uint8_t last_modifier_state;
	bool fn_key_pressed;
	bool keyboard_locked;
	
	keyboard_notify* notify;
	
	ASkeyboard();
	void hid_convert(uint8_t k, bool pressed);
	void led_demo();
};
