/**
 * 
  ps2_hobo_nicola.ino Main sketch of "Hobo-nicola PS/2 adapter" using Hobo-nicola keyboard and adapter library.
  Copyright (c) 2018 Takeshi Higasa
  
  This file is part of "Hobo-nicola keyboard and adapter".

  "Hobo-nicola keyboard and adapter" is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  "Hobo-nicola keyboard and adapter" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Hobo-nicola keyboard and adapter".  If not, see <http://www.gnu.org/licenses/>.
  
  hoboNicolaLibrary 1.5.0   Jul. 25, 2021
  
*/

#include "hobo_nicola.h"
#include "ps2_kbd.h"

// Function keys with Fn-key pressed.
static const uint16_t fn_keys[] PROGMEM = {
	HID_U_ARROW, FN_MEDIA_VOL_UP,
	HID_D_ARROW, FN_MEDIA_VOL_DOWN,
	HID_R_ARROW, FN_MEDIA_SCAN_NEXT,
	HID_L_ARROW, FN_MEDIA_SCAN_PREV,
	HID_ENTER,	 FN_MEDIA_PLAY_PAUSE,
	HID_DELETE,	 FN_MEDIA_STOP,
	HID_END,		 FN_MEDIA_MUTE,
	HID_ESCAPE | WITH_R_CTRL,	 FN_SYSTEM_SLEEP,		// Ctrl + App + Esc 
	HID_ESCAPE | WITH_R_SHIFT | WITH_R_CTRL,	 FN_SYSTEM_POWER_DOWN,
	0, 0
};

class ps2_hobo_nicola : public HoboNicola {
public:
	ps2_hobo_nicola() {}
	virtual const uint16_t* get_fn_keys() { return fn_keys; }

	virtual void nicola_led(bool on) { digitalWrite(LED_BUILTIN_RX, on ? 0 : 1); }
	virtual void toggle_nicola_led() { digitalWrite(LED_BUILTIN_RX, digitalRead(LED_BUILTIN_RX) ^ 1); }
	virtual void error_led(bool on) { digitalWrite(LED_BUILTIN_TX, on ? 0 : 1); }

};

ps2_hobo_nicola hobo_nicola;

class ps2_event : public keyboard_notify {
  public:
//    virtual void modifier_changed(uint8_t before, uint8_t after) { hobo_nicola.modifier_change(before, after); }
    virtual void modifier_changed(uint8_t before, uint8_t after) { }
    virtual void raw_key_event(uint8_t key, bool released = false) {};
    virtual void key_pressed  (uint8_t key, uint8_t mod) { hobo_nicola.key_event(key, true);  };
    virtual void key_released  (uint8_t key, uint8_t mod) { hobo_nicola.key_event(key, false); };
};

ps2_kbd* ps2;
void setup() {
	HoboNicola::init_hobo_nicola(&hobo_nicola);
  ps2 = ps2_kbd::getInstance();
  if (!ps2->begin(new ps2_event, LED_BUILTIN_RX))
    hobo_nicola.error_blink(400);
}

void loop() {
  ps2->task(hobo_nicola.get_hid_led_state());
  hobo_nicola.idle();
}
