/**
 * 
  usb_hobo_nicola.ino Main sketch of "Hobo-nicola usb adapter" using Hobo-nicola keyboard and adapter library.
  Copyright (c) 2018 Takeshi Higasa
  
  This file is part of "Hobo-nicola keyboard and adapter".

  "Hobo-nicola keyboard and adapter" is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  "Hobo-nicola keyboard and adapter" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Hobo-nicola keyboard and adapter".  If not, see <http://www.gnu.org/licenses/>.

  version 1.5.0  Jul. 20, 2021. 
	for 
*/

#include "hobo_nicola.h"
#include <hidboot.h>
#include <usbhub.h>
#include <SPI.h>

// Function keys with Fn-key pressed.
static const uint16_t fn_keys[] PROGMEM = {
	HID_U_ARROW,	FN_MEDIA_VOL_UP,
	HID_D_ARROW,	FN_MEDIA_VOL_DOWN,
	HID_R_ARROW,	FN_MEDIA_FF,
	HID_L_ARROW,	FN_MEDIA_REW,
	HID_PGUP, 		FN_MEDIA_SCAN_NEXT,
	HID_PGDOWN,		FN_MEDIA_SCAN_PREV,
	HID_ENTER,		FN_MEDIA_PLAY_PAUSE,
	HID_HOME,			FN_MEDIA_PLAY_SKIP,
	HID_DELETE,		FN_MEDIA_STOP,
	HID_END,			FN_MEDIA_MUTE,
	HID_ESCAPE | WITH_R_CTRL,	 FN_SYSTEM_SLEEP,		// Ctrl + App + Esc 
	HID_ESCAPE | WITH_R_ALT | WITH_R_CTRL,	 FN_SYSTEM_POWER_DOWN,
	HID_S			| WITH_R_CTRL , FN_SETUP_MODE,
	HID_F12	, HID_KEYPAD_NUMLOCK,	// test
	HID_F1	, HID_IME_ON,
	HID_F2	, HID_IME_OFF,
	0, 0
};

class usb_hobo_nicola : public HoboNicola {
public:
	usb_hobo_nicola() {	}
	virtual const uint16_t* get_fn_keys() { return fn_keys; }

	void modifier_change(uint8_t before, uint8_t after) {
		uint8_t change = after ^ before;
		if (!change)
			return;
		uint8_t mask = 1;
		uint8_t key = HID_MODIFIERS;
		for(uint8_t i = 0; i < 8; i++)	{
			if (change & mask)
				key_event(key, after & mask);
			mask <<= 1;
			key++;			
		}
	}
	virtual void nicola_led(bool on) { digitalWrite(LED_BUILTIN_RX, on ? 0 : 1); }
	virtual void toggle_nicola_led() { digitalWrite(LED_BUILTIN_RX, digitalRead(LED_BUILTIN_RX) ^ 1); }
	virtual void error_led(bool on) { digitalWrite(LED_BUILTIN_TX, on ? 0 : 1); }
};


usb_hobo_nicola hobo_nicola;

class KeyboardEvent : public KeyboardReportParser {
	USBHID *phid;

public:
	KeyboardEvent() { 
		phid = 0;
	};
	void Parse(USBHID *hid, bool is_rpt_id, uint8_t len, uint8_t *buf) {
		phid = hid;
		KeyboardReportParser::Parse(hid, is_rpt_id, len, buf);
	};
	
	void sync_led( uint8_t led ) {
		if (phid) phid->SetReport(0, 0, 2, 0, 1, &led);
	};
	
  protected:
	void OnControlKeysChanged(uint8_t before, uint8_t after) { hobo_nicola.modifier_change(before, after); };
	void OnKeyPressed(uint8_t key) {};
	void OnKeyDown  (uint8_t mod, uint8_t key) { hobo_nicola.key_event(key, true);  };
	void OnKeyUp  (uint8_t mod, uint8_t key) { hobo_nicola.key_event(key, false); };
};

USB Usb;
USBHub Hub1(&Usb);
USBHub Hub2(&Usb);
USBHub Hub3(&Usb);
USBHub Hub4(&Usb);
HIDBoot<USB_HID_PROTOCOL_KEYBOARD>    HidKeyboard(&Usb);
KeyboardEvent kbd;


void setup() {
	HoboNicola::init_hobo_nicola(&hobo_nicola);
	if (Usb.Init() == -1)
		hobo_nicola.error_blink();
	delay( 200 );
	HidKeyboard.SetReportParser(0, &kbd);
}

uint8_t last_led_state = 0;

void loop() {
	Usb.Task();
	hobo_nicola.idle();
	if (HoboNicola::get_hid_led_state() != last_led_state) {
		kbd.sync_led(HoboNicola::get_hid_led_state());
		last_led_state = HoboNicola::get_hid_led_state();
	}
}
