/**
 * 
  settings.cpp  settings definitions of "Hobo-nicola keyboard and adapter library".
  Copyright (c) 2021 Takeshi Higasa
  
  This file is part of "Hobo-nicola keyboard and adapter library".

  "Hobo-nicola keyboard and adapter library" is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  "Hobo-nicola keyboard and adapter library" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Hobo-nicola keyboard and adapter library".  If not, see <http://www.gnu.org/licenses/>.
  
    Included in hoboNicola 1.5.0.		Aug. 1. 2021.
*/


#ifndef __SETTINGS_H__
#define __SETTINGS_H__

#include "Arduino.h"
#include <EEPROM.h>
#define EEPROM_ADDR		0
#define EEPROM_EXTRA	8

// basic settings in EEPROM
	static const uint32_t SPC_TO_LEFT	= 1;		// 1-2のうちのいずれかだけオンにできること。
	static const uint32_t SPC_TO_RIGHT = 2;
	static const uint32_t SPC_TO_OYAYUBI = 3;	// 0x11 1-2の設定をクリアする。
	static const uint32_t OUTPUT_IMMEDIATE_ON = 4;
	static const uint32_t EISU_TO_NICOLA_OFF = 8;
	static const uint32_t KANA_TO_NICOLA_ON	= 0x10;
	static const uint32_t SINGLE_OYAYUBI_MODE = 0x40;
	static const uint32_t SELF_REPEAT = 0x80;
	static const uint32_t SCR_AS_NICOLA	= 0x100;
	static const uint32_t KANJI_TO_NICOLA_OFF	= 0x200;
	static const uint32_t KANJI_TOGGLE_NICOLA	= 0x20;		// どちらか一方だけオンにできること。
	static const uint32_t RALT_TO_HIRAGANA	= 0x400;	// 一方通行
	static const uint32_t SWAP_CAPS_CTRL		= 0x800;	// 入れ替え
	static const uint32_t US_LAYOUT = 0x2000;

// extra settings in EEPROM (EEPROM_ADDR + 8)
// rgb_value (8bit)
// some flags. (TBD).

class _Settings {
	uint32_t settings;
	uint32_t extra_settings;

	uint32_t _load(uint16_t addr) {
		uint32_t a = 0;
		uint32_t b = 0;
	
		EEPROM.get(addr, a);
		EEPROM.get(addr + 4, b);
		if (a != ~b) {
			a = 0;
			EEPROM.put(addr, a);
			EEPROM.put(addr + 4, ~a);
		}
		return a;
	}
	
	void _save(uint16_t addr, uint32_t data) {
		EEPROM.put(addr, data);
		EEPROM.put(addr + 4, ~data);
	}

public:
	_Settings() {
		load();
	}
	uint32_t get_data() { return settings; }
	uint32_t get_extra() { return extra_settings; }

	void load() {
		settings = _load(EEPROM_ADDR);
		extra_settings = _load(EEPROM_EXTRA);
	}
	void save(uint32_t new_set) {
		if (new_set == settings)
			return ;
		_save(EEPROM_ADDR, new_set);			
		settings = new_set;
	}
	void save_extra(uint32_t new_extra) {
		if (new_extra == extra_settings)
			return ;
		_save(EEPROM_EXTRA, new_extra);			
		extra_settings = new_extra;
	}


	bool is_immediate_output() const { return (bool)(settings & OUTPUT_IMMEDIATE_ON); }
	bool is_spc_to_left() const { return (bool)(settings & SPC_TO_LEFT); }
	bool is_spc_to_right() const { return (bool)(settings & SPC_TO_RIGHT); }
	bool is_spc_to_none() const { return (bool) (settings & SPC_TO_OYAYUBI) == 0; }
	bool is_self_repeat() const { return (bool)(settings & SELF_REPEAT); }

	bool is_single_oyayubi_mode() const { return (bool)(settings & SINGLE_OYAYUBI_MODE); }

	bool is_eisu_to_nicola_off() const { return (bool)(settings & EISU_TO_NICOLA_OFF); }
	bool is_kana_to_nicola_on() const { return (bool)(settings & KANA_TO_NICOLA_ON); }

	bool is_scrlock_as_nicola() const { return (bool)(settings & SCR_AS_NICOLA) == 0; }

	bool is_kanji_toggle_nicola() const { return (bool)(settings & KANJI_TOGGLE_NICOLA); }
	bool is_kanji_to_nicola_off() const { return (bool)(settings & KANJI_TO_NICOLA_OFF); }
	bool is_ralt_to_hiragana() const { return (bool)(settings & RALT_TO_HIRAGANA); }
	bool is_swap_caps_ctrl() const { return (bool)(settings & SWAP_CAPS_CTRL); }
	
	bool is_us_layout() const { return (bool)(settings & US_LAYOUT); }

};


//_Settings& Settings();

#endif //__SETTINGS_H__